import Global
import log
import rcerror
import AccessControl

class ZopeActionItem:

    def __init__(self, monitorParent, pathToMethod, argCollector):
        self.monitorItem = Global.monitor.createItem(monitorParent, "xmlrpc_exec_item")
        self.argCollector = argCollector
        self.pathToMethod = pathToMethod
        Global.log(Global.log.action, Global.log.DEBUG1,  
                        "Created ZopeActionItem with path: %s" %self.pathToMethod)
        
    def execute(self):
        app = getApplication()
        try:
            try:
                # get Zope app object
                if app is None:
                    # We are not in a Zope environment
                    # XXX maybe raise an exception?
                    return 1
                # Login as system user
                AccessControl.SecurityManagement.newSecurityManager(
                        None, AccessControl.User.system)                
                # traverse to the method to be called
                Global.log(Global.log.action, Global.log.DEBUG1,  
                            "Getting ZopeActionItem with path: %s" %self.pathToMethod)
                import sys
                if hasattr(sys,'breakpoint'):
                    sys.breakpoint()
                method = app.unrestrictedTraverse(self.pathToMethod.split('/'))
                Global.log(Global.log.action, Global.log.DEBUG1,  
                            "Got ZopeActionItem with path: %s" %self.pathToMethod)
    
                # call the method, passing the event obects
                events = self.argCollector.getEvents()
                Global.log(Global.log.action, Global.log.DEBUG1,  
                            "Calling method with events: %s" %str(events))
                
                result = method(events)
                get_transaction().commit()
                return result
            except:
                from Helpers import Helpers
                print Helpers.GetTraceback()
                Global.log(Global.log.action, Global.log.ERROR,  "Error while executing Zope method '%s'" %self.pathToMethod)
                Global.log(Global.log.action, Global.log.ERROR, "%s" %rcerror.processException())
                try:
                    get_transaction().abort()
                except AttributeError:
                    # It may be that we are not in a Zope environment
                    pass
                return 0
        finally:
            # This is VERY important when running in Zope
            releaseApplication(app)
    
try:
    # Try and import Zope stuff
    import Globals
except ImportError:
    # We are not in a Zope environment
    Globals = None

def getApplication():
    '''Get a Zope Application'''
    if Globals is not None:
        try:
            while not hasattr(Globals,'DB'):
                # Wait for Zope to wake up
                time.sleep(1)
            connection = Globals.DB.open(force=1)
            app = connection.root()['Application']
            return app
        except:
            # XXX Log the error
            raise
    else:
        return None

def releaseApplication(app):
    '''Release the Zope Application.
    This also calls get_transaction().abort() to make sure there are no uncomitted
    object left. Call get_transaction().commit() if changes must be commited!'''
    if Globals is not None and app is not None:
        try:
            get_transaction().abort()
            app._p_jar.close()
        except:
            # XXX Log the error
            raise
    else:
        return None
    
