
import sys

import rcerror
from RuleManager import RuleManager
from ConditionInstanceManager import ConditionInstanceManager
from ActionInstanceManager import ActionInstanceManager

from ConfigReader import ConfigReader
from XmlRpcEventSource import XmlRpcEventSource 
from EventStore import EventStore
from EventInput import EventInput
from EventInputServer import EventInputServer
from EventFactory import EventFactory
from EventRouter import EventRouter
from RuntimeMonitor import RuntimeMonitor
from GlobalFilterManager import GlobalFilterManager
from EventInputManager import EventInputManager
from Logger import Logger
from xml import xpath
from xml.dom.ext.reader import PyExpat
from xml.dom import implementation
from Ticker import Ticker

import logging
import signal
import SocketServer
import xmlrpcserver
import xmlrpclib
from  MonitorServer import *
import threading
import time
import Decimal
import exceptions
import log
import os


def sigHandler(sig, stack):
    
    Global.log(Global.log.main, Global.log.INFO, "ruleCore Engine shutting down on signal %s",sig)
    
    tids = threading.enumerate()
    Global.shutdown = 1
    time.sleep(1)

    tries = 20
    while tries:
        tids = threading.enumerate()
        if len(tids) == 1:
            break
        time.sleep(1)
        tries = tries - 1

    if tries:
        sys.exit(0)
    else:
        sys.exit(1)


def shutdown(arg):

    Global.shutdown = 1
    time.sleep(2)
    
    if Global.log:
        Global.log(Global.log.main, Global.log.INFO, "ruleCore Engine shutting down")
    else:
        print "ruleCore Engine shutting down"
    sys.exit(arg)

def checkFile(file):
    return os.access(file, os.R_OK)
    
    

def printUsage():

    print "ruleCore Engine version 1.0"
    print "Copyright(c) 2002-2004 MS Analog Software kb, all rights reserved"
    print "Usage: engine rcml_config_file"
    print
    print "ruleCore comes with ABSOLUTELY NO WARRANTY"
    print "For licensing options, see www.rulecore.com"


def main(configFile=None):

    try:
        if configFile is None and len(sys.argv) > 1:
            configFile = sys.argv[1]
        elif configFile is not None:
            pass
        else:
            printUsage()
            shutdown(1)
            
        if not checkFile(configFile):
            print "Could not open file '%s'" % configFile
            shutdown(2)

        config = ConfigReader(configFile)
        Global.log = Logger()
    
        startServer()

    except exceptions.SystemExit:
        print "ruleCore Engine got exit exception."
    except:
        print "Unexpected exception: %s " % rcerror.processException()    
        shutdown(3)



def startServer():
    isMain = threading.currentThread().getName() == 'MainThread'
    Global.log(Global.log.main, Global.log.INFO, "Starting ruleCore Engine version 1.0")
    Global.log(Global.log.main, Global.log.INFO, "Copyright(c) 2002-2004 MS Analog Software kb, all rights reserved")
    Global.log(Global.log.main, Global.log.INFO, "ruleCore comes with ABSOLUTELY NO WARRANTY")
    Global.log(Global.log.main, Global.log.INFO, "For licensing options, see www.rulecore.com")

    if isMain:
        signal.signal(signal.SIGHUP, sigHandler)
        signal.signal(signal.SIGQUIT, sigHandler)
        signal.signal(signal.SIGINT, sigHandler)
        signal.signal(signal.SIGTERM, sigHandler)
        signal.signal(signal.SIGUSR1, sigHandler)

    
    Global.router = EventRouter()
       
    Global.monitor = RuntimeMonitor()



    eventFactory = xpath.Evaluate("system/event-factory", Global.root)
    assert eventFactory, "No event factory"
    factory = EventFactory()
    eventFactory[0].nodeValue = factory

    ticker = Ticker()
    filterManager = GlobalFilterManager()

    inputServer = EventInputServer()

    inputManager = EventInputManager()

    input = xpath.Evaluate("system/event-input", Global.root)
    assert input, "No event input"
    eventInput = EventInput()
    input[0].nodeValue = eventInput


    stores = xpath.Evaluate("system/event-stores/child::event-store[attribute::name='default']", Global.root)
    assert stores, "No event store"
    store = EventStore()
    stores[0].nodeValue = store
    
    rules = xpath.Evaluate('config/rules', Global.root)
    assert rules, "No rules in config file"
    
    rmg = RuleManager(rules[0])

    c = ConditionInstanceManager()
    a = ActionInstanceManager()

    inputs = inputManager.getInputs()
    Global.router.addChain(inputs, [eventInput, filterManager])


    s = MonitorServer()
    s.start()

    Global.log(Global.log.main, Global.log.INFO,  "ruleCore Engine running")

    while 1:
        Global.log(Global.log.main, Global.log.DEBUG2,  "ruleCore Engine heartbeat")
        signal.pause()
        
    Global.log(Global.log.main, Global.log.INFO,  "ruleCore Engine exiting")
           



if __name__ == "__main__":
    main()


