#-----------------------------------------------------------------------------
# Name:        rulecore.py
# Purpose:     This module does some monkey patching and gives an easy way to
#              get RuleCore to run on Win32 platforms
#
# Author:      Etienne Labuschagne
#
# Created:     2004/17/03
# RCS-ID:      $Id: rulecore.py $
# Copyright:   (c) GMS 2004
# Licence:     GNU GPL
#-----------------------------------------------------------------------------

import imp
import os
import sys
import time

def _doMonkeyPatchingForWin32():
    # Add time's strptime to mx.DateTime
    try:
        from mx.DateTime import strptime
    except ImportError:
        import mx.DateTime
        mx.DateTime.strptime = time.strptime
        from mx.DateTime import strptime
    
    # Give dummy signals and methods to signal to make it work in Win32
    import signal
    signal.SIGHUP = signal.SIGQUIT = signal.SIGUSR1 = signal.SIGTERM
    # Create an infinite pause method for signal.pause
    import threading
    pauseEvent = threading.Event()
    pauseEvent.clear()
    signal.pause = pauseEvent.wait
    
    try:
        import avl
    except ImportError:
        # Could not get avl to compile on Win32 yet, instead, use the original Python implementation
        # and put avl_tree in sys.modules as avl so that import avl still works
        import avl_tree as avl
        avl.newavl = avl.avl_tree
        sys.modules['avl'] = sys.modules['avl_tree']
    
def setupEnvironment(startWhat=None):
    '''Set up the environment according to the component to start
    startWhat : the component to start ('engine','designer','monitor','evgen'). If None, then the
        first argument if sys.argv will be used'''
    startWhats = ['engine','designer','monitor','evgen']
    if startWhat is None:
        #The first parameter is what component to start
        if len(sys.argv) == 1:
            print '''Usage:
                rulecore [component] [component args]
                eg.
                main engine engineConfig.xml
                or
                main designer'''
            raise ValueError("Could not determine the component to start.")
        else:
            startWhat = sys.argv[1].lower()
            # strip out the startWhat parameter so that the args are as expected by the rulecore components
            sys.argv = [sys.argv[0]]+sys.argv[2:]
    
    # Check that we have a valid startWhat
    assert startWhat in startWhats, \
            'First parameter must be the component to start (one of %s)' %startWhats
            
    # fix up the RULECORE_HOME environment if not set
    if not os.environ.has_key('RULECORE_HOME'):
        f, fn, stuff = imp.find_module('rulecore')
        if f is not None:
            f.close()
        if not os.path.isdir(fn):
            rulecoreHome = os.path.split(fn)[0]
        else:
            rulecoreHome = fn
        os.environ['RULECORE_HOME'] = rulecoreHome
    else:
        rulecoreHome = os.environ['RULECORE_HOME']
        
    # stick the startWhat and thirdparty stuff onto the path
    sys.path.append(os.path.join(rulecoreHome,'sbin',startWhat))
    sys.path.append(os.path.join(os.environ['RULECORE_HOME'], 'thirdparty'))
    
    # monkey patch the modules that won't work under Win32
    _doMonkeyPatchingForWin32()

    # now import main - the correct main's path will be in sys.path
    import main
    if sys.argv[1:]:
        print "Environment set up for \"%s\" with arguments: %s" %(startWhat, " ".join(sys.argv[1:]))
    else:
        print "Environment set up for \"%s\" with no arguments" %(startWhat)
    return main

if __name__ == '__main__':
        # now run main
        main = setupEnvironment()
        main.main()
